# -*- coding: utf-8 -*-
"""
author: F.K. Unseld

This script generates figure 3 from the provided raw data. The indces of all 
files are in V, the data points in A.

"""


import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1.inset_locator import (inset_axes, mark_inset)
from matplotlib.ticker import EngFormatter,StrMethodFormatter
from matplotlib.ticker import MaxNLocator
from matplotlib.patches import ConnectionPatch
import os
#%%
RawDataPulsed = pd.read_csv('../Fig3/RawDataPulsed.csv')

Ref_datas = []
for Reffile in os.listdir('../Fig3/References'):
    tmp = pd.read_csv('../Fig3/References/'+Reffile)
    tmp =  tmp.set_index(tmp.columns[0])
    Ref_datas.append(tmp)

#For Legacy REasons
t780 = np.zeros((np.shape(RawDataPulsed)[0],2))
t780[:,0] = RawDataPulsed['Mean T780']
t780[:,1] = RawDataPulsed['STD T780']
t790 = np.zeros((np.shape(RawDataPulsed)[0],2))
t790[:,0] = RawDataPulsed['Mean T790']
t790[:,1] = RawDataPulsed['STD T790']
t800 = np.zeros((np.shape(RawDataPulsed)[0],2))
t800[:,0] = RawDataPulsed['Mean T800']
t800[:,1] = RawDataPulsed['STD T800']


#%%Figure (Data Only)

fig,ax = plt.subplots(nrows = 1, ncols = 1, dpi = 450, figsize=(5.4,4))

Ref_labels = ["T = 780 mV","T = 790 mV","T = 800 mV"]
linewidth = 0.7
elinewidth = 0.5
glinewidth = 0.30
marker = 'x'
markersize = 2

colrs = ['C1','C0','C2']


for (colr,(idx,Ref_data)) in zip(colrs,enumerate(Ref_datas)):
    Ref_data[Ref_data.columns[0]].plot(
        ax=ax,
        style = colr+'--',
        linewidth = 0.7, 
        label = Ref_labels[idx])
    
x = np.arange(1.500,1.550,0.5e-3)[:len(t780)]
ax.errorbar(x,
               np.array(t780)[:,0],
               yerr=np.array(t780)[:,1], 
               C = 'C1', 
               ecolor = 'C1',
               ls = '-', 
               linewidth = linewidth,
               elinewidth = elinewidth,
               marker = marker,
               markersize = markersize)
ax.errorbar(x,
               np.array(t790)[:,0],
               yerr=np.array(t790)[:,1], 
               C = 'C0', 
               ecolor = 'C0',
               ls = '-', 
               linewidth = linewidth,
               elinewidth = elinewidth,
               marker = marker,
               markersize = markersize)

ax.errorbar(x,
               np.array(t800)[:,0],
               yerr=np.array(t800)[:,1], 
               C = 'C2',
               ecolor = 'C2',
               ls = '-', 
               linewidth = linewidth,
               elinewidth = elinewidth,
               marker = marker,
               markersize = markersize)


ax.minorticks_on()
ax.grid(which='major', linestyle='--', lw = glinewidth)
ax.grid(which='minor', linestyle=':', lw = glinewidth)

ax.legend([r'Reference $T=780\mathregular{mV}$',
        r'Reference $T=790\mathregular{mV}$',
        r'Reference $T=800\mathregular{mV}$',
        r'$T=790\mathregular{mV\ (DC)}-10\mathregular{mV\ (AC)}$',
        r'$T=790\mathregular{mV\ (DC)}\ +\ 0\mathregular{mV\ (AC)}$',
        r'$T=790\mathregular{mV\ (DC)}+10\mathregular{mV\ (AC)}$',
           'Static'])


ax.yaxis.set_major_formatter(EngFormatter(unit='A'))
ax.set_ylabel(r'$\mathregular{I_d}$')
ax.xaxis.set_major_locator(MaxNLocator(nbins=7))
ax.xaxis.set_major_formatter(EngFormatter(unit='V'))
ax.set_xlabel(r'$\mathregular{V_{in}}$')


ax.set_xlim((1.5,1.54))
ax.set_ylim((-50e-12,1e-9))

ax.set_xticks(np.arange(1.5,1.54,0.01))

fig.tight_layout(h_pad=1, w_pad=1)

#%% Figure (Data and Explanation Graph)


gridsize = (3, 1)
fig = plt.figure(dpi = 450, figsize=(3.54,4))
ax1 = plt.subplot2grid(gridsize, (0, 0), colspan=1, rowspan=1)
ax2 = plt.subplot2grid(gridsize, (1, 0), colspan=1, rowspan=2)

linewidth = 0.7
style = 'butt'

ax1.set_axis_off()
Vin_y = np.array([0,0,.5,.5,1,1])
Vin_x = np.array([0,1,1,2,2,3])

Vg_y = np.array([0,0,1,1,0,0])
Vg_x = np.array([0,1,1,3,3,20])/20

T_y = np.array([1,1,2,2,0,0,2,2,0,0,2,2, 0, 0, 2, 2, 0, 0, 2, 2, 0, 0, 1,1])/2
T_x = np.array([0,5,5,6,6,7,7,8,8,9,9,10,10,11,11,12,12,13,13,14,14,15,15,20])/20

Im_xb = np.array([15,15,18,18])/20
Im_yb = np.array([np.nan,1,1,np.nan])
Im_xo = np.array([6,6,7,7,8,8,9,9,10,10,11,11,12,12,13,13,14,14,15,15])/20
Im_yo = np.array([np.nan,1,1,np.nan,np.nan,1,1,np.nan,np.nan,1,1,np.nan,np.nan,1,1,np.nan,np.nan,1,1,np.nan,])
Im_xg = np.array([5,5,6,6,7,7,8,8,9,9,10,10,11,11,12,12,13,13,14,14])/20
Im_yg = np.array([np.nan,1,1,np.nan,np.nan,1,1,np.nan,np.nan,1,1,np.nan,np.nan,1,1,np.nan,np.nan,1,1,np.nan,])
Im_xk = np.array([0,3,3,5,5,18,18,19,19,20])/20
Im_yk = np.array([0,0,1,1,np.nan,np.nan,1,1,0,0])


grln_x = np.array([5,5,5,5,6,6,6,6,7,7,7,7,8,8,8,8,9,9,9,9,10,10,10,10,11,11,11,11,12,12,12,12,13,13,13,13,14,14,14,14,15,15,15,15,18,18,18,18])/20
grln_y = np.array([np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,np.nan,-4.3,3,np.nan,])

ax1.plot(Vin_x,Vin_y+2,'k',lw= linewidth)
ax1.plot(Vg_x,Vg_y,'k',lw= linewidth)
ax1.plot(Vg_x+1,Vg_y,'k',lw= linewidth)
ax1.plot(Vg_x+2,Vg_y,'k',lw= linewidth)

ax1.plot(T_x,T_y-2,'k',lw= linewidth)
ax1.plot(T_x+1,T_y-2,'k',lw= linewidth)
ax1.plot(T_x+2,T_y-2,'k',lw= linewidth)

ax1.plot(Im_xk, Im_yk-4,'k',lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xb, Im_yb-4,lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xo, Im_yo-4,lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xg, Im_yg-4,lw= linewidth, solid_capstyle=style)

ax1.plot(Im_xk+1, Im_yk-4,'k',lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xb+1, Im_yb-4,'C0',lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xo+1, Im_yo-4,'C1',lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xg+1, Im_yg-4,'C2',lw= linewidth, solid_capstyle=style)

ax1.plot(Im_xk+2, Im_yk-4,'k',lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xb+2, Im_yb-4,'C0',lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xo+2, Im_yo-4,'C1',lw= linewidth, solid_capstyle=style)
ax1.plot(Im_xg+2, Im_yg-4,'C2',lw= linewidth, solid_capstyle=style)

ax1.plot(grln_x,grln_y,':', c = 'silver',lw=linewidth/2)

ax1.annotate(r'$\mathregular{V_{in}}$',(-.3,2.25), annotation_clip=False,size=10)
ax1.annotate(r'$\mathregular{V_{g}}$',(-.3,.25), annotation_clip=False,size=10)
ax1.annotate(r'$\mathregular{T}$',(-.3,-1.75), annotation_clip=False,size=10)
ax1.annotate(r'$\mathregular{I_{m}}$',(-.3,-3.75), annotation_clip=False,size=10)

ax1.annotate('1500 mV',(0.2,2.2), annotation_clip=False,size=5)
ax1.annotate('1501 mV',(1.2,2.7), annotation_clip=False,size=5)
ax1.annotate('1502 mV',(2.2,3.2), annotation_clip=False,size=5)

ax1.annotate('2200 mV', (4/20+2,.75), annotation_clip=False,size=5 )
ax1.annotate('1600 mV', (3+1/20,-.2), annotation_clip=False,size=5 )

ax1.annotate('800 mV',(3+1/20,-1.2), annotation_clip=False,size=5)
ax1.annotate('790 mV',(3+1/20,-1.75), annotation_clip=False,size=5)
ax1.annotate('780 mV',(3+1/20,-2.3), annotation_clip=False,size=5)

ax1.set_xlim(0,3.5)

for i,Ref_data in enumerate(Ref_datas):
    if (Ref_data.mean()<1e-6).all():
        Ref_datas[i] = Ref_data*1e9



linewidth = .5
elinewidth = .4
marker = 'x'
markersize = 0
colrs = ['C1','C0','C2']



for (colr,(idx,Ref_data)) in zip(colrs,enumerate(Ref_datas)):
    Ref_data[Ref_data.columns[0]].plot(
        ax=ax2,
        style = colr+'--',
        linewidth = 0.7, 
        label = Ref_labels[idx])
    
x = np.arange(1.500,1.550,0.5e-3)[:len(t780)]
ax2.errorbar(x,
               np.array(t780)[:,0]*1e9,
               yerr=np.array(t780)[:,1]*1e9, 
               C = 'C1', 
               ecolor = 'C1',
               ls = '-', 
               linewidth = linewidth,
               elinewidth = elinewidth,
               marker = marker,
               markersize = markersize)
ax2.errorbar(x,
               np.array(t790)[:,0]*1e9,
               yerr=np.array(t790)[:,1]*1e9, 
               C = 'C0', 
               ecolor = 'C0',
               ls = '-', 
               linewidth = linewidth,
               elinewidth = elinewidth,
               marker = marker,
               markersize = markersize)

ax2.errorbar(x,
               np.array(t800)[:,0]*1e9,
               yerr=np.array(t800)[:,1]*1e9, 
               C = 'C2',
               ecolor = 'C2',
               ls = '-', 
               linewidth = linewidth,
               elinewidth = elinewidth,
               marker = marker,
               markersize = markersize)


ax2.minorticks_on()
ax2.grid(which='major', linestyle='--', lw = 0.25)
ax2.grid(which='minor', linestyle=':', lw = 0.25)

ax2.legend([r'$T=780\mathregular{mV}$ Reference ',
       r'$T=790\mathregular{mV}$ Reference ',
       r'$T=800\mathregular{mV}$  Reference ',
       r'$T=790\mathregular{mV\ (DC)}-10\mathregular{mV\ (AC)}$',
       r'$T=790\mathregular{mV\ (DC)}\ +\ 0\mathregular{mV\ (AC)}$',
       r'$T=790\mathregular{mV\ (DC)}+10\mathregular{mV\ (AC)}$',
          'Static'],fontsize=6)


ax2.set_ylabel(r'$\mathregular{I_d}$ (nA)')
ax2.set_xlabel(r'$\mathregular{V_{in}}$ (V)')


ax2.set_xlim((1.5,1.54))
ax2.set_ylim((-51e-3,1))

ax2.set_xticks(np.arange(1.5,1.54,0.01))

con_g = ConnectionPatch(xyA=(1.5056,0.475), xyB=(5.5/20, 1-4), coordsA="data", coordsB="data",
                      axesA=ax2, axesB=ax1,arrowstyle = '<-',color = 'C2',lw = .5,mutation_scale=5.0)
con_o = ConnectionPatch(xyA=(1.5131,0.21), xyB=(8.5/20, 1-4), coordsA="data", coordsB="data",
                      axesA=ax2, axesB=ax1,arrowstyle = '<-',color = 'C1',lw = .5,mutation_scale=5.0)
con_b = ConnectionPatch(xyA=(1.5192,0.435), xyB=(16.5/20, 1-4), coordsA="data", coordsB="data",
                      axesA=ax2, axesB=ax1,arrowstyle = '<-',color = 'C0',lw = .5,mutation_scale=5.0)
ax2.add_artist(con_b)
ax2.add_artist(con_o)
ax2.add_artist(con_g)

ax2.xaxis.label.set_size(10)
ax2.yaxis.label.set_size(10)
ax2.xaxis.set_tick_params(labelsize=10)
ax2.yaxis.set_tick_params(labelsize=10)

# fig.tight_layout()
# fig.subplots_adjust(left =.15, right = .90)
plt.savefig('../Fig3/Fig3.png',bbox_inches = 'tight')
plt.savefig('../Fig3/Fig3.pdf',bbox_inches = 'tight')






