# -*- coding: utf-8 -*-
"""
author: F Unseld
This is the a script to generate Figure S5. The indices are given in Hz, the 
data in V/sqrt(Hz). 
"""

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

from matplotlib.ticker import EngFormatter
#%%
StaticRawData = pd.read_csv('../FigS5/StaticSqrtPSD.csv', header = [0], index_col=[0])
fStatic = StaticRawData.index.values
PSD_Static = StaticRawData.values
FloatingRawData = pd.read_csv('../FigS5/FloatingSqrtPSD.csv', header = [0], index_col=[0])
fFloating = FloatingRawData.index.values
PSD_Floating = FloatingRawData.values

#%% Plot
fig,ax = plt.subplots(dpi = 450, figsize=(3.54,2.5))

ax.loglog(fStatic,PSD_Static,'C0', lw = 0.5,label = 'Static')
ax.loglog(fFloating,PSD_Floating,'C1', lw = 0.5,label = 'Pulsed')
ax.set_ylabel(r'$\frac{V}{\sqrt{Hz}}$')
ax.xaxis.set_major_formatter(EngFormatter(unit='Hz'))
ax.set_xlabel('Frequency')

ax.minorticks_on()
ax.grid(which='major', linestyle='--', lw = 0.25)
ax.grid(which='minor', linestyle=':', lw = 0.25)
ax.legend()
ax.set_xlim((.03,3))


fig.tight_layout(h_pad=1, w_pad=1)
plt.savefig('../FigS5/FigS5.png',bbox_inches = 'tight')
plt.savefig('../FigS5/FigS5.pdf',bbox_inches = 'tight')






























